function [zeta_ref, eta_ee, log, switch_weight] =...
    inverseKinematics(snake,x,R,J, switch_weight)
% Uses the task priority and method for inverting jacobian matrices
% to output the reference velocity for the snake robot

pos = x(1:3);
theta = x(8:6+snake.n); %Joint angles

%Get end effector position and transformation matrix for end effector
[eta_ee,eta_n_b_ee] = getEEpos(snake, pos, theta, R);
T = getTransformationMatrix(eta_ee(4:6));

%Get position of snake base (tail)
eulerAngles = getEulerAngles(R(:,:,1));
eta = [pos; eulerAngles];

%%      Tasks
%%     Position and orientation of end effector       
eta_ee_d = [1 0 0 0 0 pi/2]';
eta_ee_d_dot = [0 0 0 0 0 0]';
K_ee = diag([0.2 0.2 0.2 0.2 0.2 0.2],0);
w_ee = eta_ee_d_dot + K_ee*(eta_ee_d - eta_ee);

J_ee = [R(:,:,snake.n)*J(1:3,:,snake.n);
        T*J(4:6,:,snake.n)];

% Saturating error term
w_ee1 = w_ee(1:3);
satee1 = abs(w_ee1) > 1;
w_ee1(satee1) = 1*sign(w_ee1(satee1));

w_ee2 = w_ee(4:6);
satee2 = abs(w_ee2) > 0.5;
w_ee2(satee2) = 0.5*sign(w_ee2(satee2));

w_ee(1:3) = w_ee1;
w_ee(4:6) = w_ee2;
    
%%    Position and orientation of base link
eta_base_d = [-2 3 0 0 0 0]';
eta_base_d_dot = [0 0 0 0 0 0]';
K_base = diag([.2 .2 .2 .2 .2 .2],0);
w_base = eta_base_d_dot + K_base*(eta_base_d - eta);

J_base = [R(:,:,1) zeros(3,2+snake.n);
          zeros(3,3) TransAngEul(eulerAngles) zeros(3,snake.n-1)];

J_base_inv = pinv(J_base);

% Activate switch when base link is close enough to target position
if (norm(eta_base_d(1:3) - eta(1:3)) < 0.2)
    switch_weight = 1;
end

% Saturating error term
w_base1 = w_base(1:3);
satbase1 = abs(w_base1) > 1;
w_base1(satbase1) = 1*sign(w_base1(satbase1));

w_base2 = w_base(4:6);
satbase2 = abs(w_base2) > 0.2;
w_base2(satbase2) = 0.2*sign(w_base2(satbase2));

w_base(1:3) = w_base1;
w_base(4:6) = w_base2;

%%              Nominal configuration
joint_d = zeros(snake.n-1,1);
joint_d_dot = zeros(snake.n-1,1);
K_nconf = 0.2;
w_nconf = joint_d_dot + K_nconf*(joint_d-theta);

J_nconf = [zeros(snake.n-1,6),eye(snake.n-1)];

sat_nconf = abs(w_nconf) > 0.1;
w_nconf(sat_nconf) = 0.1*sign(w_nconf(sat_nconf));

%%              Joint limit task
limit = pi/3;
J_joint_limit = zeros(snake.n-1,5+snake.n);
w_joint_limit = zeros(snake.n-1,1);

for i=1:(snake.n-1)
    if (theta(i) > limit)
        J_joint_limit(i,6+i) = 1;
        w_joint_limit(i) = 0.2*(limit - theta(i));
    elseif (theta(i) < -limit)
        J_joint_limit(i,6+i) = 1;
        w_joint_limit(i) = 0.2*(-limit - theta(i));
    end
end

%% ****************************************************
%%               Inverse Kinematics

%% Weighted pseudoinverse
%Mass matrix as weight
W = comp_mass(J,snake);
Wi = pinv(W,.001);

% J_ee_winv = Wi*J_ee'*pinv(J_ee*Wi*J_ee',.001);
J_base_winv = Wi*J_base'*pinv(J_base*Wi*J_base',.001);
% J_joint_limit_winv = Wi*J_joint_limit'*pinv(J_joint_limit*...
%    Wi*J_joint_limit',.001);

%% 1 task
w_1 = w_base;

J_1_inv = J_base_winv;

zeta_ref = J_1_inv*w_1;


%% 2 tasks
% w_1 = w_ee;
% w_2 = w_base;
% 
% J_1 = J_ee;
% J_2 = J_base;
% 
% J_1_inv = J_ee_winv;
% J_2_inv = J_base_winv;
% 
% N_1 = eye(5+snake.n) - J_1_inv*J_1;
% 
% zeta_ref = J_1_inv*w_1 + N_1*J_2_inv*w_2;

%% 3 tasks
% w_1 = w_joint_limit;
% w_2 = w_ee;
% w_3 = w_base;
% 
% J_1 = J_joint_limit;
% J_2 = J_ee;
% J_3 = J_base;
% 
% J_1_inv = J_joint_limit_winv;
% J_2_inv = J_ee_winv;
% J_3_inv = J_base_winv;
% 
% N_1 = eye(5+snake.n) - J_1_inv*J_1;
% 
% J_1_2 = [J_1 ; J_2];
% J_1_2_inv = pinv(J_1_2);
% N_1_2 = eye(5+snake.n) - J_1_2_inv*J_1_2;
% 
% zeta_ref = J_1_inv*w_1 + N_1*J_2_inv*w_2 + N_1_2*J_3_inv*w_3;

%% Logging tasks
log.base = eta_base_d;
log.ee = eta_ee_d;
log.nominal = joint_d;


end

